import classNames from 'classnames';
import dayjs from 'dayjs';
import React, { CSSProperties, useMemo } from 'react';
import { memo } from 'react';
import FileIcon from '../FileIcon';
import ATooltip from '../Tooltip';
import styles from './FileStatus.less';
import { CloseCircleFilled, CheckCircleFilled } from '@ant-design/icons';
import { Progress, Button } from 'antd';
import { TaskStatus, TaskType } from '@/services/API.helper';
import { DATA } from '@/services/API';
import { identity } from 'lodash';

interface FileStatusProps {
  className?: string;
  style?: CSSProperties;
  data: DATA.SocketFileMsg;
}

export default function FileStatus(props: FileStatusProps) {
  const { className, style, data, ...restProps } = props;
  console.log(data);

  const filePath = useMemo(() => {
    return [data.projName, data.nodeName, data.relativePath]
      .filter(identity)
      .join(' / ');
  }, [data]);
  const relationModifier = useMemo(() => {
    return data.modifyName
      ? `${data.modifyName}同步`
      : data.createUserName
      ? `${data.createUserName}同步`
      : '';
  }, [data]);
  return (
    <div className={classNames(styles.fileStatus, className)} style={style}>
      <div className={styles.left}>
        <FileIcon className={styles.icon} extension={data.extension} />
        <div className={styles.content}>
          <ATooltip placement="top" title={data.archName}>
            <div className={styles.fileName}>{data.archName}</div>
          </ATooltip>
          <div className={styles.subContent}>
            <ATooltip placement="bottom" title={filePath}>
              <div className={styles.filePath}>{filePath}</div>
            </ATooltip>
            <div className={styles.modifyInfo}>{relationModifier}</div>
          </div>
        </div>
      </div>
      <div className={styles.mid}>
        <LoadDesc
          type={data.taskType}
          loadingState={data.taskSyncStatus}
          progress={data.taskSyncProgress}
          {...restProps}
        />
        {data.taskSyncStatus === TaskStatus.FINISH ? (
          <Time time={'2020-01-01 18:24:56'} />
        ) : null}
      </div>
      <div className={styles.right}>
        {/* 查看1: 已下载 文件打开文件夹 */}
        {/* 查看2: 未下载 且 已删除 文件跳转到web端 */}
        {/* 下载: 未下载 且 未删除 文件 */}
        {/* 重新下载: 下载失败时出现 */}
        {/* 重新上传: 上传失败时出现 */}
        {/* <Button type="link" className={styles.button} size="small">重新上传</Button> */}
        {/* <Button type="link" className={styles.button} size="small">暂停</Button> */}
        {/* <Button type="link" className={styles.button} size="small">取消</Button> */}
      </div>
    </div>
  );
}

const Time = memo((props: { time: string }) => (
  <span className={styles.time}>{dayjs(props.time).format('HH:mm:ss')}</span>
));

interface LoadDescProps {
  type: TaskType;
  loadingState: TaskStatus;
  result?: 'success' | 'fail';
  progress?: number;
}

function LoadDesc(props: LoadDescProps) {
  const { result, type, loadingState, progress } = props;
  const keywords = type === 'UPLOAD' ? '上传' : '下载';
  const [resultText, icon] = useMemo(() => {
    if (loadingState === TaskStatus.FINISH) {
      return [
        '成功!',
        <CheckCircleFilled
          className={classNames(styles.stateIcon, styles.success)}
        />,
      ];
    }
    if (loadingState === TaskStatus.FAILED) {
      return [
        '失败!',
        <CloseCircleFilled
          className={classNames(styles.stateIcon, styles.error)}
        />,
      ];
    }
    if (loadingState === TaskStatus.WAITING) {
      return [`等待${keywords}`];
    }
    return ['', null];
  }, [result, keywords]);
  return (
    <span className={styles.loadDesc}>
      {loadingState === TaskStatus.SYNCING ? (
        <Progress percent={progress} format={() => `${keywords}中...`} />
      ) : (
        <>
          {icon}
          {keywords}
          {resultText}
        </>
      )}
    </span>
  );
}
